@@ -1,21 +1,20 @@
 from __future__ import unicode_literals
 import base64
 import os.path
-import logging
-from logging import getLogger, basicConfig
 import argparse
 
 import py
 from devpi_common.types import cached_property
+from .log import threadlog
 import devpi_server
-log = getLogger(__name__)
+from devpi_common.url import URL
+
+log = threadlog
 
 def get_default_serverdir():
     return os.environ.get("DEVPI_SERVERDIR", "~/.devpi/server")
 
 def addoptions(parser):
-
-
     web = parser.addgroup("web serving options")
     web.addoption("--host",  type=str,
             default="localhost",
@@ -33,11 +32,6 @@ def addoptions(parser):
                  "and the web server does not set or you want to override "
                  "the custom X-outside-url header.")
 
-    web.addoption("--bottleserver", metavar="TYPE",
-            default="auto",
-            help="bottle server class, tries 'eventlet', "
-                 "then 'wsgiref'  if set to 'auto' (default)")
-
     web.addoption("--debug", action="store_true",
             help="run wsgi application with debug logging")
 
@@ -58,13 +52,22 @@ def addoptions(parser):
     deploy.addoption("--version", action="store_true",
             help="show devpi_version (%s)" % devpi_server.__version__)
 
-    deploy.addoption("--gendeploy", action="store", metavar="DIR",
-            help="(unix only, deprecated) install and generate a pre-configured "
-                 "virtualenv directory which puts devpi-server "
-                 "under supervisor control and provides some example "
-                 "files for nginx/crontab.  If the specified DIR exists "
-                 "virtualenv creation will be skipped and only the "
-                 "installation steps will be performed. ")
+    deploy.addoption("--master", action="store", dest="master_url",
+            help="run as a replica of the specified master server",
+            default=None)
+
+    deploy.addoption("--replica-cert", action="store", dest="replica_cert",
+            metavar="pem_file",
+            help="when running as a replica, use the given .pem file as the "
+                 "SSL client certificate to authenticate to the server "
+                 "(EXPERIMENTAL)",
+            default=None)
+
+    deploy.addoption("--gen-config", dest="genconfig", action="store_true",
+            help="(unix only ) generate example config files for "
+                 "nginx/supervisor/crontab, taking other passed options "
+                 "into account (e.g. port, host, etc.)"
+    )
 
     deploy.addoption("--secretfile", type=str, metavar="path",
             default="{serverdir}/.secret",
@@ -74,7 +77,7 @@ def addoptions(parser):
 
     deploy.addoption("--upgrade-state", action="store_true",
             dest="upgrade_state",
-            help="upgrade server state if possible. ")
+            help="(deprecated, use --export/--import) automatically upgrade server state if possible, creating virtualenvs as needed. ")
 
     deploy.addoption("--export", type=str, metavar="PATH",
             help="export devpi-server database state into PATH. "
@@ -113,6 +116,17 @@ def addoptions(parser):
     #group.addoption("--logfile", action="store",
     #        help="set log file file location")
 
+def load_setuptools_entrypoints():
+    try:
+        from pkg_resources import iter_entry_points, DistributionNotFound
+    except ImportError:
+        return # XXX issue a warning
+    for ep in iter_entry_points('devpi_server'):
+        try:
+            plugin = ep.load()
+        except DistributionNotFound:
+            continue
+        yield plugin, ep.dist
 
 
 def try_argcomplete(parser):
@@ -123,7 +137,7 @@ def try_argcomplete(parser):
     else:
         argcomplete.autocomplete(parser)
 
-def parseoptions(argv, addoptions=addoptions):
+def parseoptions(argv, addoptions=addoptions, hook=None):
     parser = MyArgumentParser(
         description="Start a server which serves multiples users and "
                     "indices. The special root/pypi index is a real-time "
@@ -133,11 +147,13 @@ def parseoptions(argv, addoptions=addoptions):
     )
 
     addoptions(parser)
+    if hook:
+        hook.devpiserver_add_parser_options(parser)
     try_argcomplete(parser)
     raw = [str(x) for x in argv[1:]]
     args = parser.parse_args(raw)
     args._raw = raw
-    config = Config(args)
+    config = Config(args, hook=hook)
     return config
 
 class MyArgumentParser(argparse.ArgumentParser):
@@ -174,21 +190,71 @@ class MyArgumentParser(argparse.ArgumentParser):
 class ConfigurationError(Exception):
     """ incorrect configuration or environment settings. """
 
+class PluginManager:
+    def __init__(self, plugins):
+        self._plugins = plugins
+
+    def _call_plugins(self, _name, **kwargs):
+        results = []
+        for plug, distinfo in self._plugins:
+            meth = getattr(plug, _name, None)
+            if meth is not None:
+                name = "%s.%s" % (
+                            getattr(plug, "__class__", plug).__name__,
+                            meth.__name__)
+                with threadlog.around("debug", "call: %s ", name):
+                    results.append(meth(**kwargs))
+            #else:
+            #    threadlog.error("no plugin hook %s on %s", _name, plug)
+        return results
+
+    def devpiserver_pyramid_configure(self, config, pyramid_config):
+        return self._call_plugins("devpiserver_pyramid_configure",
+                                  config=config, pyramid_config=pyramid_config)
+
+    def devpiserver_add_parser_options(self, parser):
+        return self._call_plugins("devpiserver_add_parser_options",
+                                  parser=parser)
+
+    def devpiserver_run_commands(self, xom):
+        return self._call_plugins("devpiserver_run_commands",
+                                  xom=xom)
+
+    def devpiserver_on_upload(self, stage, projectname, version, link):
+        return self._call_plugins("devpiserver_on_upload",
+                                  stage=stage, projectname=projectname,
+                                  version=version, link=link)
+
+    def devpiserver_on_changed_versiondata(self, stage, projectname, version,
+                                           metadata):
+        return self._call_plugins("devpiserver_on_changed_versiondata",
+                                  stage=stage, projectname=projectname,
+                                  version=version, metadata=metadata)
+
+    def devpiserver_pypi_initial(self, stage, name2serials):
+        return self._call_plugins("devpiserver_pypi_initial",
+                                  stage=stage,
+                                  name2serials=name2serials)
+
 
 class Config:
-    def __init__(self, args):
+    def __init__(self, args, hook):
         self.args = args
+        if self.args.master_url:
+            self.master_url = URL(self.args.master_url)
         serverdir = args.serverdir
         if serverdir is None:
             serverdir = get_default_serverdir()
         self.serverdir = py.path.local(os.path.expanduser(serverdir))
 
         if args.secretfile == "{serverdir}/.secret":
-            self.secretfile = self.serverdir.join(".secret", abs=True)
+            self.secretfile = self.serverdir.join(".secret")
         else:
             self.secretfile = py.path.local(
                     os.path.expanduser(args.secretfile))
 
+        self.hook = hook
+
     @cached_property
     def secret(self):
         if not self.secretfile.check():
@@ -198,24 +264,12 @@ class Config:
             self.secretfile.chmod(s.S_IRUSR|s.S_IWUSR)
         return self.secretfile.read()
 
-def configure_logging(config):
-    if config.args.debug:
-        loglevel = logging.DEBUG
-    else:
-        loglevel = logging.INFO
-    basicConfig(level=loglevel,
-                format='%(asctime)s [%(levelname)-5.5s] %(name)s: %(message)s')
-
 def getpath(path):
     return py.path.local(os.path.expanduser(str(path)))
 
-def render(tw, basedir, confname, format=None, **kw):
+def render(tw, confname, format=None, **kw):
     result = render_string(confname, format=format, **kw)
-    conf = basedir.join(confname)
-    conf.write(result)
-    if tw is not None:
-        tw.line("wrote %s" % conf, bold=True)
-    return conf
+    return result
 
 def render_string(confname, format=None, **kw):
     template = confname + ".template"
