@@ -3,7 +3,6 @@ from __future__ import unicode_literals
 
 import hashlib
 import pytest
-import re
 import py
 import json
 import posixpath
@@ -211,7 +210,7 @@ def test_apiconfig(testapp):
     assert not "pypisubmit" in r.json["result"]
 
 def test_apiconfig_with_outside_url(testapp):
-    testapp.xom.config.args.outside_url = u = "http://outside.com/root"
+    testapp.xom.config.args.outside_url = u = "http://outside.com"
     r = testapp.get_json("/root/pypi/+api")
     assert r.status_code == 200
     result = r.json["result"]
@@ -251,6 +250,7 @@ class TestSubmitValidation:
         class Submit:
             def __init__(self, stagename="user/dev"):
                 self.stagename = stagename
+                self.username = stagename.split("/")[0]
                 self.api = mapp.create_and_use(stagename)
 
             def metadata(self, metadata, code):
@@ -272,17 +272,13 @@ class TestSubmitValidation:
         mapp.upload_file_pypi("qlwkej", b"qwe", "name", "1.0",
                               indexname="nouser/nostage", code=404)
 
-    def test_metadata_normalize_conflict(self, submit, testapp):
+    def test_metadata_normalize_to_previous_issue84(self, submit, testapp):
         metadata = {"name": "pKg1", "version": "1.0", ":action": "submit",
                     "description": "hello world"}
-        r = submit.metadata(metadata, code=200)
-        metadata = {"name": "Pkg1", "version": "1.0", ":action": "submit",
+        submit.metadata(metadata, code=200)
+        metadata = {"name": "Pkg1", "version": "2.0", ":action": "submit",
                     "description": "hello world"}
-        r = submit.metadata(metadata, code=403)
-        body = r.body
-        if not py.builtin._istext(body):
-            body = body.decode("utf-8")
-        assert re.search("pKg1.*already.*registered", body)
+        submit.metadata(metadata, code=200)
 
     def test_metadata_multifield(self, submit, mapp):
         classifiers = ["Intended Audience :: Developers",
@@ -310,7 +306,7 @@ class TestSubmitValidation:
         assert not data["download_url"]
         assert not data["platform"]
 
-    def test_upload_file(self, submit):
+    def test_upload_file(self, submit, mapp):
         metadata = {"name": "Pkg5", "version": "1.0", ":action": "submit"}
         submit.metadata(metadata, code=200)
         r = submit.file("pkg5-2.6.tgz", b"123", {"name": "pkg5some"}, code=400)
@@ -318,8 +314,9 @@ class TestSubmitValidation:
         submit.file("pkg5-2.6.tgz", b"123", {"name": "Pkg5"}, code=200)
         r = submit.file("pkg5-2.6.qwe", b"123", {"name": "Pkg5"}, code=400)
         assert "not a valid" in r.status
-        r = submit.file("pkg5-2.7.tgz", b"123", {"name": "pkg5"}, code=403)
-        assert "cannot register" in r.status
+        r = submit.file("pkg5-2.7.tgz", b"123", {"name": "pkg5"}, code=200)
+        paths = mapp.get_release_paths("Pkg5")
+        assert paths[0].endswith("pkg5-2.7.tgz")
 
     def test_upload_use_registered_name_issue84(self, submit, mapp):
         metadata = {"name": "pkg_hello", "version":"1.0", ":action": "submit"}
@@ -370,6 +367,17 @@ class TestSubmitValidation:
                 entry = testapp.xom.filestore.get_file_entry(path.strip("/"))
                 assert not entry.file_exists()
 
+    def test_upload_and_delete_user_issue130(self, submit, testapp, mapp):
+        metadata = {"name": "pkg5", "version": "2.6", ":action": "submit"}
+        submit.metadata(metadata, code=200)
+        submit.file("pkg5-2.6.tgz", b"123", {"name": "pkg5"}, code=200)
+        assert mapp.get_release_paths("pkg5")
+        mapp.delete_user(submit.username)
+        # recreate user and index
+        submit = submit.__class__(submit.stagename)
+        assert not mapp.get_release_paths("pkg5")
+
+
     def test_upload_twice_to_volatile(self, submit, testapp, mapp):
         metadata = {"name": "Pkg5", "version": "2.6", ":action": "submit"}
         submit.metadata(metadata, code=200)
@@ -776,6 +784,9 @@ def test_kvdict(input, expected):
     (
         {"X-outside-url": "http://outside.com"}, {},
         None, "http://outside.com"),
+    (
+        {"X-outside-url": "http://outside.com/foo"}, {},
+        None, "http://outside.com/foo"),
     (
         {"Host": "outside3.com"}, {},
         None, "http://outside3.com"),
@@ -792,18 +803,21 @@ def test_kvdict(input, expected):
     (
         {"X-outside-url": "http://outside.com"}, {},
         "http://outside2.com", "http://outside2.com"),
+    (
+        {"X-outside-url": "http://outside.com"}, {},
+        "http://outside2.com/foo", "http://outside2.com/foo"),
     (
         {"Host": "outside3.com"}, {},
         "http://out.com", "http://out.com"),
     (
         {"Host": "outside3.com"}, {'wsgi.url_scheme': 'https'},
         "http://out.com", "http://out.com")])
-def test_get_outside_url(headers, environ, outsideurl, expected):
-    from devpi_server.views import get_outside_url
-    from pyramid.request import Request
-    request = Request.blank('/', environ=environ, headers=headers)
-    url = get_outside_url(request, outsideurl)
-    assert url == expected
+def test_outside_url_middleware(headers, environ, outsideurl, expected, testapp):
+    headers = dict((str(k), str(v)) for k, v in headers.items())
+    environ = dict((str(k), str(v)) for k, v in environ.items())
+    testapp.xom.config.args.outside_url = outsideurl
+    r = testapp.get('/+api', headers=headers, extra_environ=environ)
+    assert r.json['result']['login'] == "%s/+login" % expected
 
 
 class Test_getjson:
